=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2008-10-25 19:26:49 +0000
+++ loggerhead/controllers/annotate_ui.py	2008-11-17 18:08:38 +0000
@@ -37,16 +37,16 @@
 
     template_path = 'loggerhead.templates.annotate'
 
-    def get_values(self, h, revid, path, kwargs, headers):
+    def get_values(self, history, revid, path, kwargs, headers):
 
-        revid = h.fix_revid(revid)
+        revid = history.fix_revid(revid)
         file_id = kwargs.get('file_id', None)
         if (file_id is None) and (path is None):
             raise HTTPBadRequest('No file_id or filename '
                                  'provided to annotate')
 
         if file_id is None:
-            file_id = h.get_file_id(revid, path)
+            file_id = history.get_file_id(revid, path)
 
         # no navbar for revisions
         navigation = util.Container()
@@ -55,6 +55,13 @@
             path = h.get_path(revid, file_id)
         filename = os.path.basename(path)
 
+        change = history.get_changes([ revid ])[0]
+        # If we're looking at the tip, use head: in the URL instead
+        if revid == history.last_revid:
+            revno_url = 'head:'
+        else:
+            revno_url = history.get_revno(revid)
+
         # Directory Breadcrumbs
         directory_breadcrumbs = (
             util.directory_breadcrumbs(
@@ -64,7 +71,7 @@
 
         # Create breadcrumb trail for the path within the branch
         try:
-            inv = h.get_inventory(revid)
+            inv = history.get_inventory(revid)
         except:
             self.log.exception('Exception fetching changes')
             raise HTTPServerError('Could not fetch changes')
@@ -72,12 +79,13 @@
 
         return {
             'revid': revid,
+            'revno_url': revno_url,
             'file_id': file_id,
             'path': path,
             'filename': filename,
             'navigation': navigation,
-            'change': h.get_changes([revid])[0],
-            'contents': list(h.annotate_file(file_id, revid)),
+            'change': change,
+            'contents': list(history.annotate_file(file_id, revid)),
             'fileview_active': True,
             'directory_breadcrumbs': directory_breadcrumbs,
             'branch_breadcrumbs': branch_breadcrumbs,

=== modified file 'loggerhead/templates/annotate.pt'
--- loggerhead/templates/annotate.pt	2008-08-16 21:53:52 +0000
+++ loggerhead/templates/annotate.pt	2008-11-17 18:08:38 +0000
@@ -36,10 +36,14 @@
                 <a tal:attributes="href python:url(['/revision'], clear=1, start_revid=change.revno)">view revision</a>
             </li>
             <li>
-                <a tal:attributes="href python:url(['/changes'], clear=1, start_revid=change.revno, filter_file_id=file_id)">view changes to this file</a>
+                <a tal:attributes="href python:url(['/changes'], clear=1, 
+                                    start_revid=change.revno, filter_file_id=file_id)"
+                >view changes to this file</a>
             </li>
             <li id="last">
-                <a tal:attributes="href python:url(['/download', change.revno, file_id, filename])">download file</a>
+                <a tal:attributes="href python:url(['/download',
+                                    revno_url, file_id, filename])"
+                >download file</a>
             </li>
         </ul>
 
@@ -47,8 +51,8 @@
         <table id="logentries">
           <tr class="logheader">
             <td class="annoLineTit">Line</td>
-            <td class="annoRevTit">Revision</td>            
-            <td class="annoContTit">Contents</td>                    
+            <td class="annoRevTit">Revision</td>
+            <td class="annoContTit">Contents</td>
           </tr>
 
           <tr tal:repeat="line contents" 

